/*  ---------------------------------------------------------------------------
    Program  :  Figure6.sas

    Project  :  Regional User Groups Graphics Presentation - 2011
    Authors  :  Watts & Derby

    Purpose  :  This is the code for FIGURE6 from the paper:
                Using SAS GTL to Visualize Your Data when there is Too Much of it to Visualize.
                It uses "rounding" (inverse of jitter) to produce a scatter plot of the
                Framingham heart data.

    Notes    :  This program is distributed on an as-is basis. To run, just change
                GPATH and LIBNAME statements so that they point to a folder on your computer.

                Updates include:
                * Change dots to squares.
                * Color the dots based on the number of ties at a given location. Eliminate
                  transparency.
                * Sort plot data by the number of ties(colorByFreq) so that the highest 
                  frequency point is plotted last - making it the most visible.
    Input    :  SASHELP.HEART
    Output   :  Figure6.png
    ------------------------------------------------------------------------  */
    ods listing gpath="C:\MySubDir";
    ods path work.templat(update) sashelp.tmplmst(read);
    ods graphics on / reset imageName="Figure6" antialiasmax=10000 imagefmt=png;

    proc template;
     define Style Fig6Style;
      parent = styles.statistical;
      class GraphFonts /
       'GraphUnicodeFont' = (", ",9pt)
       'GraphTitleFont' = (", ",11pt,bold)
       'GraphLabelFont' = (", ",9pt) 
       'GraphValueFont' = (", ",9pt)
       'GraphDataFont' = (", ",9pt);
  /* make axes more visible */
      class graphAxisLines /
        linethickness = 1px
        color=black
        contrastColor=black;
  /*get rid of chart junk associated with the "wall"*/
      class graphWalls /
        frameborder=off;    
  /*define marker attributes as a style element - rather than with a markerAttrs statement */  
      class graphDataDefault /
        linethickness=1px 
        markersize=5px 
        markersymbol="squarefilled";
  /* Create no border around the entire graph */ 
      class graphBorderLines /
        linethickness = 0px;
     end;
    run;
    ods listing style=Fig6Style image_dpi=300; 

proc template;
  define statgraph scatterhist;
  mvar N;
  dynamic XVAR YVAR XLABEL YLABEL GVAR TITLE;
  begingraph / designwidth=4.5in designheight=4.25in;
    entrytitle TITLE;
    layout lattice / rows=2 columns=2 rowweights=(.175 .825) columnweights=(.825 .175)
                     rowdatarange=union columndatarange=union rowgutter=0px columngutter=2px;
  /* histogram at X2 axis position */
      layout overlay / xaxisopts=(display=none offsetmin=0) 
                       yaxisopts=(offsetmin=0 linearopts=(tickValueList=(0 400 800) viewMax=800));
       histogram XVAR / freq=GVAR binaxis=false scale=COUNT;
      endlayout;
  /* NOBS count cell REMOVE. WE ALREADY KNOW THIS. CLUTTERS THE GRAPH */  
      layout overlay / border=TRUE; 
        entry " ";	 
      endlayout;
  /* scatter plot: Attrs stmts in CONTINUOUSLEGEND are the only way to have two different text
        sizes for values and labels in ODS. Otherwise graph and legend are controlled by
        GRAPHLABELFONT and GRAPHVALUEFONT. */
      layout overlay / xaxisopts=(offsetmax=0 label=XLABEL) yaxisopts=(offsetmax=0 label=YLABEL);	
       scatterplot y=YVAR x=XVAR / markercolorgradient=GVAR name="freq";
       continuouslegend "freq" / orient=vertical location=outside valign=bottom halign=center
                                        valueattrs=(size=7pt) titleattrs=(size=8pt)
                                        backgroundColor=GRAPHDATADEFAULT:COLOR opaque=TRUE
                                        valuecounthint=8 title="Overlay Frequencies";
 
      endlayout; 
  /* histogram at Y2 axis position */
      layout overlay / xaxisopts=(offsetmin=0 linearopts=(tickValueList=(0 400 800) viewMax=800))
                       yaxisopts=(display=none offsetmin=0);
        histogram YVAR / freq=GVAR orient=horizontal binaxis=false scale=COUNT;
      endlayout;
    endlayout;    
  endgraph;
  end;
run;

data roundHeart;
  set sashelp.heart;
  if Weight gt . and Height gt .;
  roundWeight=round(weight,2);
  roundHeight=round(height,.5);
run; 

proc sql noprint;
 select count(*) into :n from roundheart;
quit;
%let n=%left(&n);

proc sort data=roundHeart;
  by roundWeight roundHeight;
run;

proc summary data=roundHeart nway;
  class roundWeight roundHeight;
  output out=TiedObs(keep=roundWeight roundHeight _FREQ_ rename=(_FREQ_=colorByFreq));
run;

proc sort data=TiedObs out=sTiedbyObs;
  by colorByFreq;
run;

proc sgrender data=sTiedbyObs template=scatterhist;  
  dynamic YVAR="roundWeight" XVAR="roundheight" 
          YLABEL="ROUND (Weight, 2 lbs)"
          XLABEL="ROUND (Height, (*ESC*){unicode '00BD'x} Inch)"
          GVAR="colorByFreq" 
          title="Rounded Points Colored by Overlay Frequencies";
run;
