/*  ---------------------------------------------------------------------------
    Program  :  Figure8.sas

    Project  :  Regional User Groups Graphics Presentation - 2011
    Authors  :  Watts & Derby

    Purpose  :  This is the code for FIGURE8 from the paper:
                Using SAS GTL to Visualize Your Data when there is Too Much of it to Visualize.

                This program draws BMI boundary lines over the digitized KDE plot
                 to better confirm or refute "Framingham cohort appeared to be 
                 considerably overweight".
                The program also illustrates data concatenation for adding ROWS from
                 different sources to the single input data set referenced in SGRENDER.

    Notes    :  This program is distributed on an as-is basis. To run, just change
                GPATH and LIBNAME statements so that they point to a folder on your computer.

                *Since there is so much data, band plots for BMI groups would not
                 be visible.
                *formulas:
                   BMI=703*Weight/Height**2
                   weight = (BMI * height**2)/703. 
                *categories:
                   Below 18.5 = Underweight
                   18.5-<25 = Normal
                   25-<30=Overweight
                   30-high= Obese
                *When it comes to comparing with the MRW Index, the population
                 has to be broken out by gender. BMI is not gender specific, 
                 but MRW is. (MRW=Metropolitan Relative Weight - See paper)

    Input    :  Permanent SAS Data set: HeartBMI_MRW (sorted by sex in ZIP file) 
    Output   :  Figure8ALL.png, Figure8Female.png, Figure8Male.png
     ------------------------------------------------------------------------  */
    ods listing gpath="C:\MySubDir";
    libname bmi 'C:\MySubDir';
    ods path work.templat(update) sashelp.tmplmst(read);

proc template;
     define Style Figure8Style;
      parent = styles.statistical;
      class GraphFonts /
       'GraphTitleFont' = (", ",11pt,bold)
       'GraphLabelFont' = (", ",11pt) 
       'GraphValueFont' = (", ",11pt)
       'GraphDataFont' = (", ",11pt);
  /* make axes more visible */
      class graphAxisLines /
        linethickness = 1px
        color=black
        contrastColor=black;
  /*get rid of chart junk associated with the "wall"*/
      class graphWalls /
        frameborder=off;    
  /*define marker attributes as a style element - rather than with a markerAttrs statement */  
      class graphDataDefault /
        linethickness=1px 
        markersize=5px 
        markersymbol="squarefilled";
  /* Place a thin black boder border around the entire graph. */ 
      class graphBorderLines /
        linethickness = 1px
        color=black;
     end;
    run;
    ods listing style=Figure8Style image_dpi=300; 

                 
proc template;
  define statgraph TwoLegends;
  dynamic SUBSET;
  begingraph;
    entrytitle "Body Mass Index Calculated for " eval(n(BMICAT)) " " SUBSET " in the Framingham Heart Study";
    entrytitle "Superimposed Over a Digitized Contour Plot of Heights and Weights" / textAttrs=(size=10pt);
    layout overlay / xaxisopts=(label="Height" linearopts=(tickvaluelist=(50 55 60 65 70 75 80)
                                viewMin=50 viewMax=77))
                     yaxisopts=(label="Weight" linearopts=(tickvaluelist=(50 100 150 200 250 300)
                                viewMin=65 viewMax=300));	
  /* Data Source: Output from Proc KDE */    
        scatterplot 
          y=gridWeight x=gridHeight / markercolorgradient=KDEDensity name="Scatter";

       continuouslegend "Scatter" / orient=vertical location=outside valign=center halign=right
                                    valueattrs=(size=8pt) titleattrs=(size=8pt)
                                    backgroundColor=GRAPHDATADEFAULT:COLOR opaque=TRUE
                                    valuecounthint=10 title="Density";
  /* Data Source: BMILines */   
        seriesplot y=weight_bmi x=height_bmi / group=bmiGroup curveLabel=bmigroup
                                               curvelabelattrs=(size=8pt)
                                               curvelabelposition=start
                                               lineattrs=(color=black pattern=1);
  /* Data Source for Legend: work.Heart */  
        layout gridded / columns=4 autoalign=(topleft bottomleft) opaque=false border=true
                         order=rowMajor borderattrs=(color=black) columnGutter=1px pad=1px;
           entry halign=left textattrs=(weight=bold style=Italic size=8pt) "BMI Category";
           entry halign=center textattrs=(weight=bold style=Italic size=8pt) "Range";
           entry halign=right textattrs=(weight=bold style=Italic size=8pt) "n";
           entry halign=center textattrs=(weight=bold style=Italic size=8pt) "Pct";

           entry halign=left textattrs=(weight=bold size=8pt)     "Underweight:";
           entry halign=center textattrs=(weight=bold size=8pt)     "<18.5";
           entry halign=right textattrs=(weight=bold size=8pt) eval(n(underweight));
           entry halign=right textattrs=(weight=bold size=8pt)     
                 eval(round(10000*(n(underweight)/(n(BMIcat))))/100)"%";

           entry halign=left textattrs=(weight=bold size=8pt) "Normal:";
           entry halign=center textattrs=(weight=bold size=8pt) "18.5-<25";
           entry halign=right textattrs=(weight=bold size=8pt) eval(n(Normal));
           entry halign=right textattrs=(weight=bold size=8pt)
                 eval(round(10000*(n(Normal)/(n(BMIcat))))/100)"%";

           entry halign=left textattrs=(weight=bold size=8pt) "Overweight:";
           entry halign=center textattrs=(weight=bold size=8pt) "25-<30";
           entry halign=right textattrs=(weight=bold size=8pt) eval(n(overweight)); 
           entry halign=right textattrs=(weight=bold size=8pt) 
                 eval(round(10000*(n(overweight)/(n(BMIcat))))/100)"%";

           entry halign=left textattrs=(weight=bold size=8pt) "Obese:";
           entry halign=center textattrs=(weight=bold size=8pt) "30+";
           entry halign=right textattrs=(weight=bold size=8pt) eval(n(obese)); 
           entry halign=right textattrs=(weight=bold size=8pt)      
                 eval(round(10000*(n(obese)/(n(BMIcat))))/100)"%";
        endlayout;
      endlayout; 
  endgraph;
  end;
run;

  /* MAKE A DATA SET THAT COMBINES bmi.HEARTBMI_MRW AND KDE DATA. bmi.HEARTBMI_MRW 
     CONTAINS VARIABLES: ageCHDDiag Sex(Female|Male) Height Weight MRW_status
     MRWCat BMI BMICat BMIDesc
     AUTOMATICALLY=5199. "IF BMI NE ." MEANS HEIGHT WEIGHT IN THE OBSERVATION. */
  proc format;
    value BMIdscFm
       1='Underweight-Normal' 2='Normal-OverWeight' 3='Overweight-Obese';
    value BMIbinFm
      1='18.5' 2='25' 3='30';
  run;

  %macro AppendData(WhereSEX=); 
   data BMILines(keep=bmiGroup weight_bmi height_bmi);
      length BMIgroup $25;
      do i=1 to 3;
        do height_bmi=50 to 77 by 1;
          BMIgroup=put(i,BMIdscFm.);
          bmiScore=input(put(i,BMIbinFm.),best.);
          weight_bmi= (bmiScore*height_bmi**2)/703;
          output;
        end;
      end;
   run;
   data Heart(keep=Height Weight Sex BMICat Underweight Normal Overweight Obese);
    set bmi.HeartBMI_MRW;
    %if &WhereSex ne %str() %then %do; 
       if Sex eq "&WhereSex";
    %end;
     select(BMICat);
     when(1) underweight=1;
     when(2) Normal=1;
     when(3) Overweight=1;
     when(4) Obese=1;
     otherwise put 'error in SELECT stmt';
    end;
  run; 
  proc kde data=Heart;
     Bivar Height Weight / PLOTS=NONE out=KDEGridded(keep=value1 value2 density count 
                          rename=(value1=gridHeight value2=gridWeight density=KDEDensity));
  run; 
  data KDEGridded;
    set kdegridded;
    if count gt 0;
  run;
  data totalHeart;
   set heart KDEGridded bmiLines;
  run;
 %mend AppendData; 

 /* EVERYBODY */
%appendData(WhereSex=%str());
ods listing style=Figure8Style image_dpi=300; 
ods graphics on / reset imageName="Figure8ALL" antialiasmax=10000 imagefmt=png;
proc sgrender data=TotalHeart template=TwoLegends;  
  dynamic SUBSET="Subjects";
run;

 /* FEMALES*/
%appendData(WhereSex=%str(Female));
ods listing style=Figure8Style image_dpi=300; 
ods graphics on / reset imageName="Figure8Female" antialiasmax=10000 imagefmt=png;
proc sgrender data=TotalHeart template=TwoLegends;  
  dynamic SUBSET="Females";
run;

/* MALES*/
%appendData(WhereSex=%str(Male));
ods listing style=Figure8Style image_dpi=300; 
ods graphics on / reset imageName="Figure8Male" antialiasmax=10000 imagefmt=png;
proc sgrender data=TotalHeart template=TwoLegends;  
  dynamic SUBSET="Males";
run;

 
